<?php
define('PRONTUS', true);
/**
 * Invoca la API de Twitter para publicar un status.
 * Este script debe correr por CLI para funcionar.
*/

if (sizeof($argv) < 2) {
    exit;
}

# Capturo los argumentos
$ts = $argv[1];
$file_url = $argv[2];
$titular_tw = base64_decode($argv[3]);
$imagen_tw = base64_decode($argv[4]);
$prontus_id = $argv[5];
$protocol = $argv[6];
$server_name = $argv[7];
$document_root = $argv[8];

require_once dirname(__FILE__) . '/config.php';
require_once dirname(__FILE__) . '/Utiles.class.php';
require_once dirname(__FILE__) . '/tmhOAuth-0.8.4/tmhOAuth.php';

main($file_url, $titular_tw, $imagen_tw, $protocol, $server_name, $document_root);

// función global para preparar el post a twitter.
function main($file_url, $titular_tw, $imagen_tw, $protocol, $server_name, $document_root) {
    $status = Utiles::clearTags($titular_tw);

    // Se obtiene o procesa la URL corta+
    $file_url = $protocol.'://' . $server_name . $file_url;

    if (defined("BITLY_KEY")) {
        $short_url = Utiles::bitlyUrl($file_url);
    } else if (defined("GOOGL_KEY")) {
        $short_url = Utiles::googleShortener($file_url);
    } else {
        $short_url = $file_url;
    }

    // Se procesa la cuenta
    $response = updateStatus($status, $short_url, $imagen_tw, TWITTER_ID, $document_root);
    echo json_encode(setMessageResult($response));
}

/**
 * Ejecuta el posting a twitter.
 * @param string $titular
 * @param string $short_url,
 * @param string $imagen_tw,
 * @param string $cuenta_twitter,
 * @param string $document_root
 * @return array
 */
function updateStatus($titular, $short_url, $imagen_tw, $cuenta_twitter, $document_root) {
    // Se calcula el Largo que debe tener el titular
    //$maxtitu = 140 - (mb_strlen($short_url, 'utf-8') + 1); // se suma 1 por el espacio antes de la url
    // usando el acortador de twitter en t.co el titular final tendra 22 caracteres.
    $maxtitu = 257; // 280 - 22 - 1
    // Se comprueba que tan largo es el titular
    if (mb_strlen($titular, 'utf-8') > $maxtitu) {
        $titular_cortado = mb_substr($titular, 0, $maxtitu - 7, 'utf-8');
        $titular_cortado = mb_substr($titular_cortado, 0, mb_strrpos($titular_cortado, ' ', 'utf-8'), 'utf-8');
        $titular = $titular_cortado . '&ellip;';
    }

    // Se le agrega la URL al final del titular
    $status = $titular . ' ' . $short_url;

    // Se leen los parámetros para autenticarse
    list($OAUTH_TOKEN, $OAUTH_TOKEN_SECRET, $CONSUMER_KEY, $CONSUMER_SECRET) = Utiles::getTwitterConf($cuenta_twitter);
    $tmhOAuth = new tmhOAuth(array(
        'consumer_key'          => $CONSUMER_KEY,
        'consumer_secret'       => $CONSUMER_SECRET,
        'user_token'            => $OAUTH_TOKEN,
        'user_secret'           => $OAUTH_TOKEN_SECRET,
        'curl_ssl_verifypeer'   => false
    ));

    fwrite(STDERR, 'imagen_tw: '.$imagen_tw. PHP_EOL);
    if ($imagen_tw != 'none') {
        // Se indica la ruta completa de la imagen
        $path_imagen = $document_root.$imagen_tw;
        $image = file_get_contents($path_imagen);

        // Se envía el status más la imagen
        $tmhOAuth->request('POST', 'https://api.twitter.com/1.1/statuses/update_with_media.json',
            array(
                'media[]'  => $image,
                'status'   => $status
            ),
            true, // use auth
            true  // multipart
        );
    } else {
        // Se envía el status sin la imagen
        $tmhOAuth->request('POST', 'https://api.twitter.com/1.1/statuses/update.json',
            array(
                'status'   => $status
            ),
            true, // use auth
            false  // multipart
        );
    }
    return $tmhOAuth->response;
}

/**
 * Recibe un array devuelto por la API de twitter y genera un array de mensaje para Prontus.
 * @param array $response
 * @return array
 */
function setMessageResult($response) {
    $message = array("msg"=>"", "status"=>0, "time" => date('d/m/Y h:m:s'));

    if ($response['code'] == '200') {
        if ($response['error'] !== '') {
            $message["msg"] = $response['error'];
        } else {
            $jsonresp = $response['response'];
            $json = json_decode($jsonresp, TRUE);

            $id_str = (!empty($json["id_str"])) ? $json["id_str"] : '';
            $message["msg"] = "Tweet publicado con ID #" . $id_str;
            $message['status'] = 1;
        }
    } else {
        $message["msg"] = 'Error al intentar publicar, código ['.$response['code'].'], error['.$response['error'].']';
    }

    return $message;
}
